@@ -1,6 +1,6 @@
 (ns com.gfredericks.test.chuck.regexes
   "Internals of the string-from-regex generator."
-  (:require [clojure.set :as set]
+  (:require [clojure.java.io]
             [clojure.test.check.generators :as gen]
             [com.gfredericks.test.chuck.regexes.charsets :as charsets]
             [instaparse.core :as insta]))
@@ -42,9 +42,8 @@
 (defn ^:private re?
   "Checks if the string compiles with re-pattern."
   [s]
-  (try (re-pattern s)
-       true
-       (catch java.util.regex.PatternSyntaxException e
+  (try (and (re-pattern s) true)
+       (catch java.util.regex.PatternSyntaxException _e
          false)))
 
 (def ^:private features
@@ -61,7 +60,17 @@
                    b1
                    (binding [*out* *err*]
                      (println "Warning: test.chuck is confused.")
-                     false)))})
+                     false)))
+   :grapheme-cluster (re? "\\X")
+   :named-characters (re? "\\N{LATIN CAPITAL LETTER X}")})
+
+(def ^:private code-point-of
+  "Like Character/codePointOf, except always returns nil if we're on
+  java 8 or something and the method doesn't even exist."
+  (try
+    (eval '#(Character/codePointOf %))
+    (catch Exception _e
+      (constantly nil))))
 
 (defn ^:private analyze-range
   [begin end]
@@ -79,9 +88,18 @@
                (when-let [[c1 & [c2 :as cs]] (seq chars)]
                  (if (= \\ c1)
                    (if (= \Q c2)
-                     (remove-QE-quoting (rest cs))
+                     (remove-QE-quoting-init (rest cs))
                      (list* c1 c2 (remove-QE-not-quoting (rest cs))))
                    (cons c1 (remove-QE-not-quoting cs))))))
+            ;; I don't understand why this clause in the java code,
+            ;; but it is, and this is what it does, and it has some
+            ;; weird effects, like in #"\c\Q0"
+            (remove-QE-quoting-init [chars]
+              (when-let [[c1 :as cs] (seq chars)]
+                (if (#{\0 \1 \2 \3 \4 \5 \6 \7 \8 \9} c1)
+                  (list* \\ \x \3 c1
+                         (remove-QE-quoting (rest cs)))
+                  (remove-QE-quoting cs))))
             (remove-QE-quoting [chars]
               (lazy-seq
                (when-let [[c1 & [c2 :as cs]] (seq chars)]
@@ -102,6 +120,19 @@
   {:type :unsupported,
    :unsupported #{feature}})
 
+(defn ^:private code-point-literal
+  [code-point]
+  (if (< code-point 0x10000)
+    {:type :character
+     :character (char code-point)}
+    {:type :large-unicode-character
+     :code-point code-point}))
+
+(defn ^:private large-unicode-character->string
+  [code-point]
+  (str (Character/highSurrogate code-point)
+       (Character/lowSurrogate code-point)))
+
 (defn analyze
   [parsed-regex]
   (insta/transform
@@ -114,26 +145,26 @@
                               :elements (->> regexes
                                              (remove nil?)
                                              (remove #{:flags}))}
-                             (some #{:flags} regexes)
-                             (assoc :unsupported #{:flags})))
+                       (some #{:flags} regexes)
+                       (assoc :unsupported #{:flags})))
     :MutatingMatchFlags (constantly :flags)
     :SuffixedExpr (fn
                     ([regex] regex)
                     ([regex {:keys [bounds quantifier]}]
-                       (cond-> {:type     :repetition
-                                :elements [regex]
-                                :bounds   bounds}
-                               quantifier
-                               (assoc :unsupported
-                                 #{:quantifiers}))))
+                     (cond-> {:type     :repetition
+                              :elements [regex]
+                              :bounds   bounds}
+                       quantifier
+                       (assoc :unsupported
+                              #{:quantifiers}))))
     :Suffix (fn
               ;; this function can get a nil 2nd or 3rd arg because of
               ;; DanglingCurlyRepetitions, which we don't hide so we
               ;; get more parse error coverage, e.g. for #"{1,0}"
               [bounds & [quantifier]]
               (cond-> {:bounds bounds}
-                      quantifier
-                      (assoc :quantifier quantifier)))
+                quantifier
+                (assoc :quantifier quantifier)))
     :Optional (constantly [0 1])
     :Positive (constantly [1 nil])
     :NonNegative (constantly [0 nil])
@@ -141,32 +172,37 @@
                        ([s] (let [n (parse-bigint s)] [n n]))
                        ([s _comma] [(parse-bigint s) nil])
                        ([s1 _comma s2]
-                          (let [lower (parse-bigint s1)
-                                upper (parse-bigint s2)]
-                            (when (< upper lower)
-                              (throw (ex-info "Bad repetition range"
-                                              {:type ::parse-error
-                                               :range [lower upper]})))
-                            [lower upper])))
+                        (let [lower (parse-bigint s1)
+                              upper (parse-bigint s2)]
+                          (when (< upper lower)
+                            (throw (ex-info "Bad repetition range"
+                                            {:type ::parse-error
+                                             :range [lower upper]})))
+                          [lower upper])))
     ;; the only reason we don't hide this in the parser is to force
     ;; the "Bad reptition range" check above, so that our "parses
     ;; exactly what re-pattern does" spec will pass.
     :DanglingCurlyRepetitions (constantly nil)
     :ParenthesizedExpr (fn
                          ([alternation]
-                            {:type :group
-                             :elements [alternation]})
+                          {:type :group
+                           :elements [alternation]})
                          ([group-flags alternation]
-                          (cond-> {:type :group
-                                   :elements [alternation]
-                                   :flag group-flags}
-                            ;; If the flags are just an empty
-                            ;; NonCapturingMatchFlags, we can safely
-                            ;; ignore it; this should correspond to
-                            ;; something like #"foo(?:bar)*"
-                            (not= group-flags
-                                  [:GroupFlags [:NonCapturingMatchFlags [:MatchFlagsExpr]]])
-                            (assoc :unsupported #{:flags}))))
+                          (let [parsed {:type :group
+                                        :elements [alternation]
+                                        :flag group-flags}]
+                            ;; :NonCapturingMatchFlags corresponds to #"foo(?:bar)*"
+                            ;; :NamedCapturingGroup corresponds to #"foo(?<x>bar)"
+                            ;; We can just ignore these for generation.
+                            (if (or
+                                 ;; e.g., #"(?:bar)" (no flags)
+                                 (= group-flags
+                                    [:GroupFlags [:NonCapturingMatchFlags [:MatchFlagsExpr]]])
+                                 ;; e.g., #"(?<x>bar)"
+                                 (let [[_flag-header [flag-type _flag-details]] group-flags]
+                                   (= :NamedCapturingGroup flag-type)))
+                              parsed
+                              (assoc parsed :unsupported #{:flags})))))
     :SingleExpr identity
     :LinebreakMatcher (constantly
                        {:type :alternation
@@ -178,6 +214,12 @@
                                    {:type :class
                                     :simple-class :R}]
                         :feature :backslash-R})
+    :GraphemeClusterMatcher (constantly
+                             {:type :class
+                              :simple-class "X"
+                              ;; this feature looks hard to support
+                              :unsupported #{"Unicode extended grapheme cluster matcher"}
+                              :feature :grapheme-cluster})
     :BaseExpr identity
     :CharExpr identity
     :LiteralChar identity
@@ -189,6 +231,18 @@
                    ;; it must be right.
                    {:type :character, :character (-> c int (bit-xor 64) char)})
 
+    :NamedChar (fn [s]
+                 (if-let [code-point (try
+                                       (code-point-of s)
+                                       (catch IllegalArgumentException _e
+                                         false))]
+                   (assoc
+                    (code-point-literal code-point)
+                    :feature :named-characters)
+                   (throw (ex-info "Bad character name"
+                                   {:type ::parse-error
+                                    :character-name s}))))
+
     ;; sounds super tricky. looking forward to investigating
     :Anchor (constantly (unsupported :anchors))
 
@@ -196,8 +250,9 @@
     :SpecialCharClass (fn [[c]]
                         (cond-> {:type :class
                                  :simple-class c}
-                                (#{\V \h \H} c)
-                                (assoc :feature :HV-classes)))
+
+                          (#{\V \h \H} c)
+                          (assoc :feature :HV-classes)))
 
     ;; If we want to support these do we need to be able to detect ungenerateable
     ;; expressions such as #"((x)|(y))\2\3"?
@@ -207,17 +262,17 @@
            (cond-> {:type :class
                     :elements [intersection]
                     :brackets? true}
-                   dangling-ampersands
-                   (assoc :undefined #{:dangling-ampersands})))
+             dangling-ampersands
+             (assoc :undefined #{:dangling-ampersands})))
     :BCCIntersection (fn [& unions]
                        (cond-> {:type :class-intersection
                                 :elements unions}
-                               ;; This could be supported, but it's
-                               ;; pretty weird so I'm giving up for
-                               ;; now.
-                               (> (count unions) 1)
-                               (assoc :unsupported
-                                 #{"Character class intersections"})))
+                         ;; This could be supported, but it's
+                         ;; pretty weird so I'm giving up for
+                         ;; now.
+                         (> (count unions) 1)
+                         (assoc :unsupported
+                                #{"Character class intersections"})))
     :BCCUnionLeft (fn [& els]
                     (let [[negated? els] (if (= "^" (first els))
                                            [true (rest els)]
@@ -225,14 +280,13 @@
                       {:type :class-union
                        :elements
                        (if negated?
-                         [(cond->
-                           {:type :class-negation
-                            :elements [{:type :class-union
-                                        :elements els}]}
-                           ;; the jvm's behavior here seems pretttty weird:
-                           ;; compare #"[^[x]]" and #"[^[x]x]"
-                           (some :brackets? els)
-                           (assoc :undefined #{"Character classes nested in negations"}))]
+                         [(cond-> {:type :class-negation
+                                   :elements [{:type :class-union
+                                               :elements els}]}
+                            ;; the jvm's behavior here seems pretttty weird:
+                            ;; compare #"[^[x]]" and #"[^[x]x]"
+                            (some :brackets? els)
+                            (assoc :undefined #{"Character classes nested in negations"}))]
                          els)}))
     :BCCNegation identity
     :BCCUnionNonLeft (fn self [& els]
@@ -249,19 +303,19 @@
                           :elements els}))
     :BCCElemHardLeft (fn self
                        ([x]
-                          (if (= x "]")
-                            {:type :character, :character \]}
-                            x))
-                       ([amps x]
-                          (update-in (self x)
-                                     [:undefined]
-                                     (fnil conj #{})
-                                     "Leading double ampersands")))
+                        (if (= x "]")
+                          {:type :character, :character \]}
+                          x))
+                       ([_amps x]
+                        (update-in (self x)
+                                   [:undefined]
+                                   (fnil conj #{})
+                                   "Leading double ampersands")))
     :BCCElemLeft identity
     :BCCElemNonLeft identity
     :BCCElemBase (fn [x] (if (= :character (:type x))
-                           {:type :class-base, :chars #{(:character x)}}
-                           x))
+                          {:type :class-base, :chars #{(:character x)}}
+                          x))
     :BCCRange analyze-range
     :BCCRangeWithBracket #(analyze-range {:type :character, :character \]} %)
     :BCCChar identity
@@ -277,7 +331,7 @@
                                 :character (normal-slashed-characters c)})
 
     :BasicEscapedChar (fn [[c]] {:type :character
-                                 :character c})
+                                :character c})
 
     :HexChar (fn [^String hex-string]
                (let [n (BigInteger. hex-string 16)]
@@ -285,12 +339,7 @@
                    (throw (ex-info "Bad hex character!"
                                    {:type ::parse-error
                                     :hex-string hex-string})))
-                 (if (> n 16rFFFF)
-                   {:type :large-unicode-character
-                    :unsupported #{"large unicode hex literals"}
-                    :n n}
-                   {:type :character
-                    :character (char (int n))})))
+                 (code-point-literal (int n))))
     :ShortHexChar identity
     :MediumHexChar identity
     :LongHexChar identity
@@ -302,7 +351,7 @@
     :OctalDigits2 list
     :OctalDigits3 list
 
-    :UnicodeCharacterClass (fn [p name]
+    :UnicodeCharacterClass (fn [_p name]
                              ;; TODO: this is not a complete list
                              (if (#{"C" "L" "M" "N" "P" "S" "Z"
                                     "{Lower}" "{Upper}" "{ASCII}"
@@ -336,8 +385,12 @@
       (case (:type m)
         :range
         (let [[m1 m2] (:elements m)
-              c1 (or (some-> m1 :character int) (:n m1))
-              c2 (or (some-> m2 :character int) (:n m2))]
+              element->codepoint (fn [x]
+                                   (case (:type x)
+                                     :character (-> x :character int)
+                                     :large-unicode-character (:code-point x)))
+              c1 (or (some-> m1 element->codepoint) (:n m1))
+              c2 (or (some-> m2 element->codepoint) (:n m2))]
           (when (and (integer? c1) (integer? c2) (< c2 c1))
             (throw (ex-info "Bad character range"
                             {:type ::parse-error
@@ -351,7 +404,7 @@
                               {:type ::parse-error
                                :text s})))))
         nil)
-      (if-let [f (:feature m)]
+      (when-let [f (:feature m)]
         (when-not (features f)
           (throw (ex-info "Regex feature not supported on this jvm!"
                           {:type ::parse-error
@@ -362,7 +415,7 @@
   passed to analyzed->generator."
   [s]
   (let [preprocessed (remove-QE s)
-        [the-parse & more :as ret] (insta/parses the-parser preprocessed)]
+        [the-parse & _more :as ret] (insta/parses the-parser preprocessed)]
     (cond (nil? the-parse)
           (throw (ex-info "Parse failure!"
                           {:type ::parse-error
@@ -435,6 +488,10 @@
   [m]
   (charsets/singleton (str (:character m))))
 
+(defmethod compile-class :large-unicode-character
+  [m]
+  (charsets/singleton (large-unicode-character->string (:code-point m))))
+
 (defmulti analyzed->generator :type)
 
 (defmethod analyzed->generator :default
@@ -465,6 +522,10 @@
   {:pre [character]}
   (gen/return (str character)))
 
+(defmethod analyzed->generator :large-unicode-character
+  [{:keys [code-point]}]
+  (gen/return (large-unicode-character->string code-point)))
+
 (defmethod analyzed->generator :repetition
   [{:keys [elements bounds]}]
   (let [[lower upper] bounds]
