@@ -8,8 +8,8 @@
   \"Accept-Encoding\")."
   (:require [clojure.string :as s]
             [potemkin :as potemkin])
-  (:import (java.util Locale)
-           (org.apache.http Header HeaderIterator)))
+  (:import java.util.Locale
+           [org.apache.http Header HeaderIterator]))
 
 (def special-cases
   "A collection of HTTP headers that do not follow the normal
@@ -119,9 +119,14 @@
         mta)
   (with-meta [_ mta]
     (HeaderMap. m mta))
+
   clojure.lang.Associative
   (containsKey [_ k]
                (contains? m (normalize k)))
+  (entryAt [_ k]
+           (if (contains? m (normalize k))
+             (clojure.lang.MapEntry. k (get _ k))))
+
   (empty [_]
          (HeaderMap. {} nil)))
 
@@ -131,13 +136,19 @@
   (into (HeaderMap. {} nil)
         (apply array-map keyvals)))
 
+(defn- header-map-request
+  [req]
+  (let [req-headers (:headers req)]
+    (if req-headers
+      (-> req (assoc :headers (into (header-map) req-headers)
+                     :use-header-maps-in-response? true))
+      req)))
+
 (defn wrap-header-map
   "Middleware that converts headers from a map into a header-map."
   [client]
-  (fn [req]
-    (let [req-headers (:headers req)
-          req (if req-headers
-                (-> req (assoc :headers (into (header-map) req-headers)
-                               :use-header-maps-in-response? true))
-                req)]
-      (client req))))
+  (fn
+    ([req]
+     (client (header-map-request req)))
+    ([req respond raise]
+     (client (header-map-request req) respond raise))))
