@@ -1,18 +1,18 @@
 (ns puppetlabs.puppetdb.meta
   (:require [clojure.tools.logging :as log]
             [puppetlabs.puppetdb.middleware :as mid]
-            [puppetlabs.trapperkeeper.core :refer [defservice]]
-            [puppetlabs.kitchensink.core :as kitchensink]
-            [slingshot.slingshot :refer [try+]]
             [puppetlabs.puppetdb.http :as http]
             [puppetlabs.puppetdb.meta.version :as v]
             [puppetlabs.puppetdb.time :refer [now]]
             [puppetlabs.puppetdb.config :as conf]
+            [puppetlabs.ring-middleware.core :as rmc]
             [puppetlabs.comidi :as cmdi]
             [bidi.schema :as bidi-schema]
             [puppetlabs.puppetdb.schema :as pls]
             [schema.core :as s]
-            [puppetlabs.i18n.core :refer [trs tru]]))
+            [puppetlabs.i18n.core :refer [trs tru]])
+  (:import
+   (clojure.lang ExceptionInfo)))
 
 (defn current-version-fn
   "Returns a function that always returns a JSON object with the running
@@ -41,16 +41,16 @@
           (http/json-response {:newer false
                                :version (v/version)
                                :link nil})
-          (try+
-           (http/json-response (v/update-info update-server scf-read-db))
-           (catch map? {m :message}
-             (log/debug m
-                        (trs "Could not retrieve update information ({0})"
-                             update-server))
-             (http/error-response (tru "Could not find version") 404))))
+          (try
+            (http/json-response (v/update-info update-server scf-read-db))
+            (catch ExceptionInfo ex
+              (log/debug (:message (ex-data ex))
+                         (trs "Could not retrieve update information ({0})"
+                              update-server))
+              (http/error-response (tru "Could not find version") 404))))
 
         (catch java.io.IOException e
-          (log/debug e (trs "Error when checking for latest version") )
+          (log/debug e (trs "Error when checking for latest version"))
           (http/error-response
            (tru "Error when checking for latest version: {0}" (.getMessage e))))))))
 
@@ -70,5 +70,5 @@
   [get-shared-globals config]
   (-> (meta-routes get-shared-globals config)
       mid/make-pdb-handler
-      mid/verify-accepts-json
+      rmc/wrap-accepts-json
       mid/validate-no-query-params))
