@@ -7,6 +7,7 @@
             [puppetlabs.puppetdb.utils.metrics :refer [multitime!]]
             [puppetlabs.puppetdb.http :as http]
             [puppetlabs.puppetdb.schema :refer [defn-validated]]
+            [puppetlabs.ssl-utils.core :refer [get-cn-from-x509-certificate]]
             [ring.util.request :as request]
             [clojure.string :as str]
             [clojure.tools.logging :as log]
@@ -24,7 +25,9 @@
             [puppetlabs.puppetdb.constants :as constants]
             [puppetlabs.puppetdb.command.constants :as const])
   (:import
-   (java.net HttpURLConnection)))
+   (clojure.lang ExceptionInfo)
+   (java.net HttpURLConnection)
+   (java.sql SQLException)))
 
 (def handler-schema (s/=> s/Any {s/Any s/Any}))
 
@@ -70,7 +73,7 @@
   the key's value is set to nil."
   [app]
   (fn [{:keys [ssl-client-cert] :as req}]
-    (let [cn  (some-> ssl-client-cert kitchensink/cn-for-cert)
+    (let [cn  (some-> ssl-client-cert get-cn-from-x509-certificate)
           req (assoc req :ssl-client-cn cn)]
       (app req))))
 
@@ -97,8 +100,20 @@
   (fn [req]
     (try
       (app req)
+      ;; This clause may become obsolete if everything it was handling
+      ;; should be (and eventually is) converted to bad-query-ex.
       (catch IllegalArgumentException e
-        (http/error-response e)))))
+        (http/error-response e))
+      (catch ExceptionInfo ex
+        (case (-> ex ex-data :kind)
+          :puppetlabs.puppetdb.query/invalid
+          (do ;; cf. produce-streaming-body
+            (cond
+              (seq (.getSuppressed ex)) (log/error ex)
+              (ex-cause ex) (log/info (ex-cause ex) (trs "Invalid query: {0}" (ex-message ex)))
+              :else (log/info (trs "Invalid query: {0}" (ex-message ex))))
+            (http/error-response ex))
+          (throw ex))))))
 
 (defn cause-finder
   [ex]
@@ -112,34 +127,41 @@
   (fn [req]
     (try
       (app req)
+      (catch ExceptionInfo e
+        (if (= :puppetlabs.puppetdb.query/terminated (:kind (ex-data e)))
+          (http/error-response (tru "Query backend terminated")
+                               HttpURLConnection/HTTP_INTERNAL_ERROR)
+          (do
+            (log/error e)
+            (http/error-response (cause-finder e)
+                                 HttpURLConnection/HTTP_INTERNAL_ERROR))))
+      (catch SQLException e
+        (cond
+          (= (.getSQLState e) (jdbc/sql-state :admin-shutdown))
+          (do
+            (if (seq (.getSuppressed e))
+              (log/error e)
+              (log/error (.getMessage e)))
+            (http/error-response (.getMessage e)
+                                 HttpURLConnection/HTTP_UNAVAILABLE))
+          :else
+          (do
+            (log/error e)
+            (http/error-response (cause-finder e)
+                                 HttpURLConnection/HTTP_INTERNAL_ERROR))))
       (catch Exception e
         (log/error e)
         (http/error-response (cause-finder e)
                              HttpURLConnection/HTTP_INTERNAL_ERROR))
-     (catch AssertionError e
+      (catch AssertionError e
         (log/error e)
         (http/error-response (tru "An unexpected error occurred while processing the request")
                              HttpURLConnection/HTTP_INTERNAL_ERROR))
-     (catch Throwable e
+      (catch Throwable e
         (log/error e)
         (http/error-response (tru "An unexpected error occurred")
                              HttpURLConnection/HTTP_INTERNAL_ERROR)))))
 
-(defn verify-accepts-content-type
-  "Ring middleware that requires a request for the wrapped `app` to accept the
-  provided `content-type`. If the content type isn't acceptable, a 406 Not
-  Acceptable status is returned, with a message informing the client it must
-  accept the content type."
-  [app content-type]
-  {:pre [(string? content-type)]}
-  (fn [{:keys [headers] :as req}]
-    (if (http/acceptable-content-type
-         content-type
-         (headers "accept"))
-      (app req)
-      (http/error-response (tru "must accept {0}" content-type)
-                           HttpURLConnection/HTTP_NOT_ACCEPTABLE))))
-
 (defn verify-content-encoding
   "Verification for the specified list of content-encodings."
   [app allowed-encodings]
@@ -155,22 +177,6 @@
                                   content-encoding)
                              HttpURLConnection/HTTP_UNSUPPORTED_TYPE)))))
 
-(defn verify-content-type
-  "Verification for the specified list of content-types."
-  [app content-types]
-  {:pre [(coll? content-types)
-         (every? string? content-types)]}
-  (fn [{:keys [headers] :as req}]
-    (if (= (:request-method req) :post)
-      (let [content-type (headers "content-type")
-            mediatype (if (nil? content-type) nil
-                          (kitchensink/base-type content-type))]
-        (if (or (nil? mediatype) (some #{mediatype} content-types))
-          (app req)
-          (http/error-response (tru "content type {0} not supported" mediatype)
-                               HttpURLConnection/HTTP_UNSUPPORTED_TYPE)))
-      (app req))))
-
 (def params-schema {(s/optional-key :optional) [s/Str]
                     (s/optional-key :required) [s/Str]})
 
@@ -211,14 +217,6 @@
   [app]
   (validate-query-params app {}))
 
-(def verify-accepts-json
-  "Ring middleware which requires a request for `app` to accept
-  application/json as a content type. If the request doesn't accept
-  application/json, a 406 Not Acceptable status is returned with an error
-  message indicating the problem."
-  (fn [app]
-    (verify-accepts-content-type app "application/json")))
-
 (def http-metrics-registry (get-in metrics/metrics-registries [:http :registry]))
 
 (defn wrap-with-metrics
@@ -357,7 +355,8 @@
         (if (= (get-sync-ver) maybe-sync-ver)
           (app req)
           (http/error-response
-           (tru "Conflicting PDB sync versions, each PDB syncing must be on the same version")
+           (tru "PDB sync request version {0} too new for this server (expected {1})."
+                maybe-sync-ver (get-sync-ver))
            409))
 
         (:error maybe-sync-ver)
