@@ -6,7 +6,6 @@
             [puppetlabs.i18n.core :refer [trs tru]]
             [clojure.tools.logging :as log]
             [schema.core :as s]
-            [clojure.data :as data]
             [puppetlabs.puppetdb.schema :as pls]
             [puppetlabs.puppetdb.archive :as archive]
             [clojure.java.io :as io]
@@ -19,10 +18,22 @@
    [java.nio ByteBuffer CharBuffer]
    [java.nio.channels SocketChannel]
    [java.nio.charset Charset CoderResult StandardCharsets]
+   (java.sql SQLException)
    (java.util.concurrent ScheduledThreadPoolExecutor TimeUnit)
    (org.apache.log4j MDC)
    (org.eclipse.jetty.io SocketChannelEndPoint)))
 
+(defn leaf-error? [ex]
+  (cond
+    (.getCause ex) false
+    (seq (.getSuppressed ex)) false
+    (and (instance? SQLException ex) (.getNextException ex)) false
+    :else true))
+
+(defn known-error? [ex]
+  (and (instance? ExceptionInfo ex)
+       (-> ex ex-data :puppetlabs.puppetdb/known-error?)))
+
 (defmacro with-captured-throw [& body]
   `(try [(do ~@body)] (catch Throwable ex# ex#)))
 
@@ -39,9 +50,10 @@
     (flush)))
 
 (defmacro with-log-mdc
-  "Establishes the MDC contexts given by the alternating-kvs key value
-  pairs during the execution of the body, and ensures that the
-  original values (if any) are always restored before returning."
+  "Establishes the MDC contexts given by the alternating-kvs key value pairs
+  during the execution of the body, and ensures that the original values (if
+  any) are always restored before returning.  The keys associated with any nil
+  values will be removed via MDC/remove."
   [alternating-kvs & body]
   ;; For now, assume this isn't used in performance-critical code,
   ;; i.e. within tight loops.
@@ -55,18 +67,18 @@
       expansion
       (recur alternating-kvs
              ;; We know k is a string, so it's fine to repeat ~k
-             `(let [v# ~v]
+             `(let [v# ~v
+                    orig# (MDC/get ~k)]
+                ;; After you put a nil value MDC/getContext crashes
                 (if (nil? v#)
+                  (MDC/remove ~k)
+                  (MDC/put ~k v#))
+                (try
                   ~expansion
-                  (let [orig# (MDC/get ~k)]
-                    (try
-                      (MDC/put ~k v#)
-                      ~expansion
-                      (finally
-                        ;; After you put a nil value MDC/getContext crashes
-                        (if (nil? orig#)
-                          (MDC/remove ~k)
-                          (MDC/put ~k orig#)))))))))))
+                  (finally
+                    (if (nil? orig#)
+                      (MDC/remove ~k)
+                      (MDC/put ~k orig#)))))))))
 
 (defn flush-and-exit
   "Attempts to flush *out* and *err*, reporting any failures to *err*,
@@ -83,19 +95,6 @@
     (catch Exception _ nil))
   (System/exit status))
 
-(pls/defn-validated diff-fn
-  "Run clojure.data/diff on `left` and `right`, calling `left-only-fn`, `right-only-fn` and `same-fn` with
-   the results of the call. Those functions should always receive a non-nil argument (though possibly empty)."
-  [left :- {s/Any s/Any}
-   right :- {s/Any s/Any}
-   left-only-fn :- pls/Function
-   right-only-fn :- pls/Function
-   same-fn :- pls/Function]
-  (let [[left-only right-only same] (data/diff (kitchensink/keyset left) (kitchensink/keyset right))]
-    (left-only-fn (or left-only #{}))
-    (right-only-fn (or right-only #{}))
-    (same-fn (or same #{}))))
-
 (def tar-item {(s/optional-key :msg) String
                :file-suffix [String]
                :contents String})
@@ -554,7 +553,7 @@
 (defn await-scheduler-shutdown [s wait-time]
   (.awaitTermination s wait-time TimeUnit/MILLISECONDS))
 
-(defn schedule [s f delay]
+(defn schedule [s f ^long delay]
   (.schedule s f delay TimeUnit/MILLISECONDS))
 
 (defn schedule-at-fixed-rate [s f initial-delay period]
